/*
 * main.c
 *
 *  Created on: 18-03-2012
 *      Author: Andrzej Michalec
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include "bitmacros.h"

#define OUT_POWER_KEEP			PB4
#define LED_SWITCH				PB3
#define LED_R					PB1
#define LED_G					PB2
#define LED_B					PB0
#define MAX_BRIGHTNESS			15

#define RGB(r, g, b) ( (uint16_t)( ((0x000F & r) << 8) | ((0x000F & g) << 4) | (0x000F & b)) )
#define R(word) ((0x0F00 & word) >> 8)
#define G(word) ((0x00F0 & word) >> 4)
#define B(word) (0x000F & word)

// Current phase
volatile uint8_t g_pwm_phase;

// Array of brightness for RGB components of LED
volatile uint8_t g_rgb[3];

const uint8_t rgbpin[] = { LED_R, LED_G, LED_B };

void set_rgb(uint16_t rgb) {
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
		g_rgb[0] = R(rgb);
		g_rgb[1] = G(rgb);
		g_rgb[2] = B(rgb);
	}
}


/**
 * LED PWM modulation.
 */
ISR(TIM0_OVF_vect)
{
	TCNT0 = 0xFF - 20;
	for (uint8_t color = 0; color < 3; color++) {
		if (g_pwm_phase < g_rgb[color]) {
			SET_BIT(PORTB, rgbpin[color]);
		} else {
			CLEAR_BIT(PORTB, rgbpin[color]);
		}
	}
	g_pwm_phase = (g_pwm_phase + 1) % (MAX_BRIGHTNESS + 1);
}

int main(void) {
	DDRB = 0xFF;
	PORTB = 0x00;
	// tact switch must be hold some time before start
	_delay_ms(200);
	// now drive T1 to keep power after releasing tact switch
	SET_BIT(PORTB, OUT_POWER_KEEP);
	// timer with f/64 prescaler
	SET_MASK(TCCR0B, BIT(CS01) | BIT(CS00));
	// enable overflow interruption
	SET_BIT(TIMSK0, TOIE0);
	// enable global interruptions
	sei();
	// blinking
	for (uint16_t i = 0; i < 3000; i++) {
		uint8_t r = (i / 7) % 32;
		if (r > 15) {
			r = 31 - r;
		}
		uint8_t g = (i / 5) % 32;
		if (g > 15) {
			g = 31 - g;
		}
		uint8_t b = (i / 3) % 32;
		if (b > 15) {
			b = 31 - b;
		}
		set_rgb(RGB(r,g,b));
		_delay_ms(10);
	}
	// done, fading out
	for (int i = 0; i < 15; i++) {
		for (int color = 0; color < 3; color++) {
			if (g_rgb[color] > 0)
				g_rgb[color]--;
		}
		_delay_ms(100);
	}

	// disabling interruptions to cut of LED driver
	cli();
	// disabling timer
	CLEAR_BIT(TIMSK0, TOIE0);
	// cutting off power and wait in loop to die :)
	CLEAR_BIT(PORTB, OUT_POWER_KEEP);
	while (1);
}
